/*
 
  copyright (c) 2004, Michal Salaban <emes@pld-linux.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2 as
  published by the Free Software Foundation (see file COPYING for details).

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
*/

/* $Id: sysfence.c,v 1.4 2004/02/08 19:07:13 emes Exp $ */

#include <stdio.h>
#include "exit.h"
#include "parseopt/lex.h"
#include "conditions.h"
#include "parseopt/parse.h"
#include "parseopt/confread.h"
#include "mainloop.h"

const char      *usage  =
                "Usage: sysfence <config file> [<config file> ...] \n"
                "\n"
                "http://sysfence.sourceforge.net/\n"
                "Report bugs to <emes@pld-linux.org>\n"
                "\n";

#ifdef DEBUG
void print_atomic (atomic *a)
{
    if (a->type == DOUBLE) {
        printf ("(%0.2f ", *((double *)a->val));
        printf (tok2string (a->op));
        printf (" %0.2f)",  *((double *)a->thresh));
    } else if (a->type == SIZE) {
        printf ("(%d ", *((long int *)a->val));
        printf (tok2string (a->op));
        printf (" %d)",  *((long int *)a->thresh));
    }
}

void print_expr (expression *e)
{
    if (e->type == ATOMIC) {
        print_atomic ((atomic *) e->arg1);
    } else {
        printf (" (");
        print_expr ((expression *) e->arg1);
        printf (" %s ", tok2string (e->op));
        print_expr ((expression *) e->arg2);
        printf (") ");
    }
}
#endif

int main (int argc, char *argv[])
{
#ifdef DEBUG
    ruleset     *rl;
    int         i;
#endif
    
    if (argc < 2) bail_out (EXIT_NOCONF, NULL);

    ruletab = read_config_files (argc - 1, &argv[1]);
    if (*ruletab == NULL) bail_out (EXIT_NORULE, NULL);
    
#ifdef DEBUG
    i = 0;
    rl = *(ruletab + i);
    while (rl != NULL) {
        printf ("\nRULESET %d:\n", i);
        print_expr (rl->expr);
        printf (" == %d\n", check_expression (rl->expr));
        if (rl->log != NULL) {
            printf ("log");
            if ((rl->log)->once) printf (" once");
            printf ("\n");
        }
        if (rl->run != NULL) {
            printf ("run '%s'", (rl->run)->runcmd);
            if ((rl->run)->once) printf (" once");
            printf ("\n");
        }
        printf ("RULESET-END\n\n");
        i ++;
        rl = *(ruletab + i);
    }
#endif

    open_files ();

#ifndef DEBUG
    if (daemon (0, 0) == 0) main_loop ();
    else bail_out (EXIT_IO, "daemon()");
#else
    main_loop ();
#endif
    
    return 0;
}

/* $Id: sysfence.c,v 1.4 2004/02/08 19:07:13 emes Exp $ */
